package com.sun.security.sasl;

import java.io.IOException;
import javax.security.auth.callback.UnsupportedCallbackException;
import java.security.NoSuchAlgorithmException;
import java.io.UnsupportedEncodingException;
import java.util.Random;
import javax.security.sasl.AuthorizeCallback;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.NameCallback;
import java.util.logging.Level;
import javax.security.sasl.SaslException;
import java.util.Map;
import javax.security.auth.callback.CallbackHandler;
import javax.security.sasl.SaslServer;

final class CramMD5Server extends CramMD5Base implements SaslServer
{
    private String fqdn;
    private byte[] challengeData;
    private String authzid;
    private CallbackHandler cbh;
    
    CramMD5Server(String s, String fqdn, Map<String, ?> map, CallbackHandler cbh) throws SaslException {
        this.challengeData = null;
        if (fqdn == null) {
            throw new SaslException("CRAM-MD5: fully qualified server name must be specified");
        }
        this.fqdn = fqdn;
        this.cbh = cbh;
        return;
    }
    
    @Override
    public byte[] evaluateResponse(byte[] array) throws SaslException {
        if (this.completed) {
            throw new IllegalStateException("CRAM-MD5 authentication already completed");
        }
        if (this.aborted) {
            throw new IllegalStateException("CRAM-MD5 authentication previously aborted due to error");
        }
        try {
            if (this.challengeData == null) {
                if (array.length != 0) {
                    this.aborted = true;
                    throw new SaslException("CRAM-MD5 does not expect any initial response");
                }
                long nextLong = new Random().nextLong();
                long currentTimeMillis = System.currentTimeMillis();
                StringBuffer sb = new StringBuffer();
                sb.append('<');
                sb.append(nextLong);
                sb.append('.');
                sb.append(currentTimeMillis);
                sb.append('@');
                sb.append(this.fqdn);
                sb.append('>');
                String string = sb.toString();
                CramMD5Server.logger.log(Level.FINE, "CRAMSRV01:Generated challenge: {0}", string);
                this.challengeData = string.getBytes("UTF8");
                return (byte[])this.challengeData.clone();
            }
            else {
                if (CramMD5Server.logger.isLoggable(Level.FINE)) {
                    CramMD5Server.logger.log(Level.FINE, "CRAMSRV02:Received response: {0}", new String(array, "UTF8"));
                }
                int length = 0;
                int i = 0;
                while (i < array.length) {
                    if (array[i] == 32) {
                        length = i;
                        break;
                    }
                    ++i;
                }
                if (length == 0) {
                    this.aborted = true;
                    throw new SaslException("CRAM-MD5: Invalid response; space missing");
                }
                String o = new String(array, 0, length, "UTF8");
                CramMD5Server.logger.log(Level.FINE, "CRAMSRV03:Extracted username: {0}", o);
                NameCallback o2 = new NameCallback("CRAM-MD5 authentication ID: ", o);
                PasswordCallback o3 = new PasswordCallback("CRAM-MD5 password: ", false);
                this.cbh.handle(new Callback[] { o2, o3 });
                char[] password = o3.getPassword();
                if (password == null || password.length == 0) {
                    this.aborted = true;
                    throw new SaslException("CRAM-MD5: username not found: " + o);
                }
                o3.clearPassword();
                String o4 = new String(password);
                for (int j = 0; j < password.length; ++j) {
                    password[j] = '\0';
                }
                this.pw = o4.getBytes("UTF8");
                String hmac_MD5 = CramMD5Base.HMAC_MD5(this.pw, this.challengeData);
                CramMD5Server.logger.log(Level.FINE, "CRAMSRV04:Expecting digest: {0}", hmac_MD5);
                this.clearPassword();
                byte[] bytes = hmac_MD5.getBytes("UTF8");
                if (bytes.length != array.length - length - 1) {
                    this.aborted = true;
                    throw new SaslException("Invalid response");
                }
                int n = 0;
                int k = length + 1;
                while (k < array.length) {
                    if (bytes[n++] != array[k]) {
                        this.aborted = true;
                        throw new SaslException("Invalid response");
                    }
                    ++k;
                }
                AuthorizeCallback o5 = new AuthorizeCallback(o, o);
                this.cbh.handle(new Callback[] { o5 });
                if (o5.isAuthorized()) {
                    this.authzid = o5.getAuthorizedID();
                    CramMD5Server.logger.log(Level.FINE, "CRAMSRV05:Authorization id: {0}", this.authzid);
                    this.completed = true;
                    return null;
                }
                this.aborted = true;
                throw new SaslException("CRAM-MD5: user not authorized: " + o);
            }
        }
        catch (UnsupportedEncodingException ex) {
            this.aborted = true;
            throw new SaslException("UTF8 not available on platform", ex);
        }
        catch (NoSuchAlgorithmException ex2) {
            this.aborted = true;
            throw new SaslException("MD5 algorithm not available on platform", ex2);
        }
        catch (UnsupportedCallbackException ex3) {
            this.aborted = true;
            throw new SaslException("CRAM-MD5 authentication failed", ex3);
        }
        catch (SaslException ex4) {
            throw ex4;
        }
        catch (IOException ex5) {
            this.aborted = true;
            throw new SaslException("CRAM-MD5 authentication failed", ex5);
        }
    }
    
    @Override
    public String getAuthorizationID() {
        if (this.completed) {
            return this.authzid;
        }
        throw new IllegalStateException("CRAM-MD5 authentication not completed");
    }
}
